<?php

namespace Drupal\ark;

use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Service for easily getting information about content citation mapping support.
 *
 * @package Drupal\ark
 */
class ArkEnabledContentTypeInformation {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * Static cache of access policies; keyed by entity type.
   *
   * @var array
   */
  protected $cache = [];

  /**
   * Constructs a new ARK-enabled Content Type Information service.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EntityFieldManagerInterface $entity_field_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entity_field_manager;
  }

  public function getContentFieldOptions($entity_type_id, $bundle, $field_type) {
    // $fields = \Drupal::service('entity_field.manager')->getFieldDefinitions('node', $bundle);
    $fields = $this->entityFieldManager->getFieldDefinitions($entity_type_id, $bundle);

    $options = [];
    foreach ($fields as $field_name => $field_info) {
      /** @var \Drupal\Core\Field\FieldDefinitionInterface $field_info */
      if ($field_info->getType() === $field_type) {
        $options[$field_name] = $field_info->getLabel();
      }
    }
    // Sort options alphabetically.
    asort($options);

    return $options;
  }

  /**
   * Determine whether the entity type is supported.
   *
   * @return bool
   *   TRUE if the entity type is supported; FALSE otherwise.
   */
  public function isSupportedEntityType(EntityTypeInterface $entity_type) {
    $supports_bundles = $entity_type->getBundleEntityType();
    $has_view_builder = ($entity_type->get('field_ui_base_route') && $entity_type->hasViewBuilderClass());
    $id_valid = $this->isIdKeyValid($entity_type);
    if ($entity_type instanceof ContentEntityTypeInterface && !$entity_type->isInternal() && $supports_bundles && $has_view_builder && $id_valid && !$this->isExcluded($entity_type)) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Exclude certain core entity types.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The current entity type.
   *
   * @return bool
   *   TRUE if the entity type is supported; FALSE otherwise.
   */
  private function isExcluded(EntityTypeInterface $entity_type) {
    // @TODO support more than nodes
    // $excluded = ['comment'];
    // return in_array($entity_type->id(), $excluded);
    return $entity_type->id() !== 'node';
  }

  /**
   * Determine whether the entity id key is valid.
   *
   * This is a workaround for some contrib modules which, for some reason,
   * don't use an integer for content entity type unique ids. Ideally we should
   * check the field type but that's not possible without causing an infinite
   * loop.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type.
   *
   * @return bool
   *   TRUE if the id is valid; FALSE otherwise.
   */
  private function isIdKeyValid(EntityTypeInterface $entity_type) {
    return (substr_count($entity_type->getKey('id'), 'id') > 0);
  }

  /**
   * Reset the static cache.
   */
  public function resetCache() {
    $this->cache = [];
  }

}
