<?php
namespace Drupal\ark;

class ArkSettings extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ark_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('ark.settings');

    foreach (Element::children($form) as $variable) {
      $config->set($variable, $form_state->getValue($form[$variable]['#parents']));
    }
    $config->save();

    if (method_exists($this, '_submitForm')) {
      $this->_submitForm($form, $form_state);
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['ark.settings'];
  }

  public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state) {

    $form['ark_naan'] = [
      '#type' => 'textfield',
      '#title' => t('Name Assigning Authority Number'),
      '#description' => t('Before your ARK identifiers can be shared publicly, a NAAN must be provided to you by the <a href="http://www.cdlib.org/">California Digital Library</a>. See <a href="https://wiki.ucop.edu/display/Curation/ARK#ARK-NAAN:theNameAssigningAuthorityNumber">the ARK wiki for more information</a>. Use "pending" here to test your site before you have a number.'),
      '#default_value' => variable_get('ark_naan', 'pending'),
      '#size' => 10,
      '#required' => TRUE,
    ];

    $form['ark_name_prefix'] = [
      '#type' => 'textfield',
      '#title' => t('Prefix for ARK names'),
      '#description' => t('Every name assigning authority must have a namespace management strategy. A time-honored technique is to hierarchically partition a namespace into subnamespaces using prefixes that guarantee non-collision of names in different partition. This practice is strongly encouraged for all NAAs, especially when subnamespace management will be delegated to other departments, units, or projects within an organization.'),
      '#default_value' => variable_get('ark_name_prefix', ark_default_prefix()),
      '#size' => 10,
    ];

    $form['ark_ezid_user'] = [
      '#type' => 'textfield',
      '#title' => t('Username for the University of California Digital Library'),
      '#description' => t('Username provided by University of California EZID to use the API system'),
      '#default_value' => variable_get('ark_ezid_user'),
      '#size' => 30,
    ];

    $form['ark_ezid_password'] = [
      '#type' => 'password',
      '#title' => t('Password for the University of California Digital Library'),
      '#description' => t('Password provided by University of California EZID to use the API system'),
      '#default_value' => variable_get('ark_ezid_password'),
      '#size' => 30,
    ];

    $form['ark_remote_server_address'] = [
      '#type' => 'textfield',
      '#title' => t('Remote ARK server address. Normally the server address for the University of California Digital Library'),
      '#description' => t('Server address for the University of California EZID to use the API system'),
      '#default_value' => variable_get('ark_remote_server_address', ARK_DEFAULT_REMOTE_SERVER_ADDRESS),
      '#size' => 90,
    ];

    $form['ark_remote_identifier_site_address'] = [
      '#type' => 'textfield',
      '#title' => t('Remote Identifier Site Address. By default "https://n2t.net/"'),
      '#description' => t('Remote Identifier Site Address. The remote Identifiear as URL will composed by this server url + the Remote Ark Name value. Ex: https://n2t.net/ark:/99999/fk4pecetest15'),
      '#default_value' => variable_get('ark_remote_identifier_site_address', ARK_DEFAULT_REMOTE_IDENTIFIER_SITE_ADDRESS),
      '#size' => 90,
    ];


    $node_types = ark_all_available_node_types();
    $form['ark_node_types'] = [
      '#type' => 'checkboxes',
      '#multiple' => TRUE,
      '#title' => t('Content types'),
      '#description' => t('Select the content types in which every piece of content should receive Archival Resource Key identifiers.'),
      '#options' => $node_types,
      '#default_value' => variable_get('ark_node_types', ''),
    ];

    $form['fields_explanation'] = [
      '#markup' => '<p>' . t('Select the fields to use for Who (a textfield) and When (a datefield) for each resource/content type. (What is always matched to the content title.)') . '</p>'
      ];

    foreach ($node_types as $type_machine_name => $type_human) {
      $who = 'ark_' . $type_machine_name . '_who';

      $form['fs-' . $type_machine_name] = [
        '#type' => 'fieldset',
        '#title' => $type_human,
      ];
      $form['fs-' . $type_machine_name][$who] = [
        '#type' => 'select',
        '#title' => t('Who'),
        '#options' => ark_fields_by_type_by_bundle('node', $type_machine_name, 'text'),
        '#description' => t('The @entity_type @bundle needs a @field_type field to be used. If no other option being choose, the @bundle Author will be used.', [
          '@entity_type' => 'node',
          '@bundle' => $type_machine_name,
          '@field_type' => 'text',
        ]),
        '#default_value' => variable_get($who, ARK_SELECT_DEFAULT_WHO_OPTION_VALUE),
      ];

      $compositeWhoFields = pece_ark_check_if_composite_who_fields_exists_in_content_type($type_machine_name);
      if ($compositeWhoFields) {
        $composite_who_options = [];
        foreach ($compositeWhoFields as $compositeWhoField) {
          $composite_who_options[$compositeWhoField] = $compositeWhoField;
        }

        $composite_who = 'ark_' . $type_machine_name . '_composite_who';
        $default_value = variable_get($composite_who);
        $checked_all = (count(array_unique(array_values($default_value))) == 1) ? [
          'checked' => 'checked'
          ] : [];

        $form['fs-' . $type_machine_name][$composite_who] = [
          '#title' => t('Use fields bellow as a composite field for WHO'),
          '#type' => 'checkboxes',
          '#description' => t('Choose composite WHO fields.'),
          '#options' => $composite_who_options,
          '#default_value' => $default_value,
          '#attributes' => $checked_all,
          '#tree' => TRUE,
          '#states' => [
            'visible' => [
              'select[name="' . $who . '"]' => [
                'value' => 'use_composite_who'
                ]
              ]
            ],
        ];
      }

      $when = 'ark_' . $type_machine_name . '_when';
      $form['fs-' . $type_machine_name][$when] = [
        '#type' => 'select',
        '#title' => t('When'),
        '#options' => ark_fields_by_type_by_bundle('node', $type_machine_name, 'date'),
        '#description' => t('The @entity_type @bundle needs a @field_type field to be used. If no other option being choose, the @bundle creation date will be used.', [
          '@entity_type' => 'node',
          '@bundle' => $type_machine_name,
          '@field_type' => 'date',
        ]),
        '#default_value' => variable_get($when, ARK_SELECT_DEFAULT_WHEN_OPTION_VALUE),
      ];

    }

    $form['#validate'][] = 'ark_settings_validate';
    $form['#submit'][] = 'pece_ark_settings_submit';

    return parent::buildForm($form, $form_state);
  }

  public function validateForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
    if (!ark_naan_format_valid($form_state->getValue(['ark_naan']))) {
      $form_state->setErrorByName('ark_naan', t('NAAN must be a number, or, if you do not yet have one, test the site using "pending".'));
    }
    if (!$form_state->getValue(['ark_node_types'])) {
      $form_state->setErrorByName('ark_node_types', t('You must select at least one content type to use as a resource.'));
    }
    else {
      foreach ($form_state->getValue(['ark_node_types']) as $type_machine_name => $type_human) {
        if (!$form_state->getValue(['ark_' . $type_machine_name . '_who'])) {
          $form_state->setErrorByName('ark_' . $type_machine_name . '_who', t('There must be a text field on the selected %type_human content type used for "Who".', [
            '%type_human' => $type_human
            ]));
        }
        if (!$form_state->getValue(['ark_' . $type_machine_name . '_when'])) {
          $form_state->setErrorByName('ark_' . $type_machine_name . '_when', t('There must be a date field on the selected %type_human content type used for "When".', [
            '%type_human' => $type_human
            ]));
        }
      }
    }
  }

}
