<?php

namespace Drupal\ark\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;

/**
 * Represents an ARK Enabled Content Type.
 *
 * @ConfigEntityType(
 *   id = "ark_content_type",
 *   label = @Translation("ARK Enabled Content Type"),
 *   label_collection = @Translation("ARK Enabled Content Types"),
 *   label_singular = @Translation("ARK-enabled content type"),
 *   label_plural = @Translation("ARK-enabled content types"),
 *   label_count = @PluralTranslation(
 *     singular = "@count ARK-enabled content type",
 *     plural = "@count ARK-enabled content types",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\ark\ArkEnabledContentTypeListBuilder",
 *     "form" = {
 *       "add" = "Drupal\ark\Form\ArkEnabledContentTypeCreateForm",
 *       "edit" = "Drupal\ark\Form\ArkEnabledContentTypeEditForm",
 *       "delete" = "Drupal\ark\Form\ArkEnabledContentTypeDeleteForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider"
 *     },
 *   },
 *   config_prefix = "ark",
 *   admin_permission = "administer ark",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "target_entity_type_id",
 *     "target_bundle",
 *   },
 *   links = {
 *     "add-form" = "/admin/config/services/ark/content/add",
 *     "edit-form" = "/admin/config/services/ark/content/{type}",
 *     "delete-form" = "/admin/config/services/ark/content/{type}/delete",
 *     "collection" = "/admin/config/services/ark/content"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "target_entity_type_id",
 *     "target_bundle",
 *     "who_fields",
 *     "when_field",
 *   }
 * )
 */
class ArkEnabledContentType extends ConfigEntityBase {

  /**
   * Unique machine name of the content type.
   *
   * @var string
   */
  protected $id;

  /**
   * The ARK-enabled content type label.
   *
   * @var string
   */
  protected $label;

  /**
   * Fields that map to the ARK who.
   *  
   * who “told” it (similar to DC Creator, Contributor, and Publisher, but also Inventor, Discoverer, Conductor, etc).
   * 
   * We allow for multiple authors/contributors, potentially coming from multiple fields on content.
   * 
   * An array of strings representing the field machine names.
   * 
   * @var array
   */
  protected $who_fields;

  /**
   * Field that maps to the ARK when.
   *
   * Must be a date field.  String representing the field machine name.
   * 
   * @var string
   */
  protected $when_field;

  /**
   * {@inheritdoc}
   */
  public function label() {
    return $this->label;
  }

  /**
   * The target entity type.
   *
   * @var string
   */
  protected $target_entity_type_id;

  /**
   * The target bundle.
   *
   * @var string
   */
  protected $target_bundle;


  /**
   * {@inheritdoc}
   */
  public function getTargetEntityTypeId() {
    return $this->target_entity_type_id;
  }

  /**
   * {@inheritdoc}
   */
  public function getTargetEntityType() {
    return \Drupal::service('entity_type.manager')->getDefinition($this->target_entity_type_id);
  }

  /**
   * {@inheritdoc}
   */
  public function setTargetEntityTypeId($entity_type_id) {
    $this->target_entity_type_id = $entity_type_id;
  }

  /**
   * {@inheritdoc}
   */
  public function getTargetBundle() {
    return $this->target_bundle;
  }

  /**
   * {@inheritdoc}
   */
  public function setTargetBundle($bundle) {
    $this->target_bundle = $bundle;
  }

  /**
   * Get who fields for this content type.
   *
   * An array of strings representing the field machine names.
   * 
   * @return array
   */
  public function getWhoFields() {
    return $this->who_fields;
  }

  /**
   * Set field mapping.
   *
   * @param array $field_map
   *   A field map, content field machine name to reference field machine name.
   */
  public function setWhoFields(array $who_fields) {
    $this->who_fields = $who_fields;
  }

  /**
   * {@inheritdoc}
   */
  public function getWhenField() {
    return $this->when_field;
  }

  /**
   * {@inheritdoc}
   */
  public function setWhenField($when_field) {
    $this->reference_type = $when_field;
  }

}
