<?php

namespace Drupal\ark\Form;

use Drupal\ark\ArkEnabledContentTypeInformation;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ArkEnabledContentTypeCreateForm.
 *
 * The Content CitationMap create form.
 *
 * @package Drupal\ark\Form
 */
class ArkEnabledContentTypeCreateForm extends EntityForm {

  /**
   * The content citation mapping information service.
   *
   * @var \Drupal\ark\ArkEnabledContentTypeInformation
   */
  protected $arkContentTypeInfo;

  /**
   * Constructs a ArkEnabledContentTypeCreateForm object.
   *
   * @param \Drupal\ark\ArkEnabledContentTypeInformation $ark_content_type_info
   *   The content citation mapping information service.
   */
  public function __construct(ArkEnabledContentTypeInformation $ark_content_type_info) {
    $this->arkContentTypeInfo = $ark_content_type_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('ark.content_type_info'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $form['target_entity_type_id'] = [
      '#type' => 'select',
      '#title' => $this->t('Entity type'),
      '#default_value' => 'node',
      '#options' => $this->getContentEntityTypeOptions(),
      '#description' => $this->t('Choose the entity type that this citation mapping is for.'),
    ];

    $form['target_bundle'] = [
      '#type' => 'select',
      '#title' => $this->t('Bundle'),
      '#default_value' => '',
      '#options' => $this->getContentBundleOptions(),
      '#description' => $this->t('Choose the bundle (for example, content type) that this citation mapping is for.'),
    ];

    return $form;
  }

  /**
   * Get all the supported content entity types.
   *
   * @return array
   *   Array of content entity types options, keyed by entity type id.
   */
  private function getContentEntityTypeOptions() {
    $entity_types = $this->entityTypeManager->getDefinitions();
    $content_entity_types = array_filter($entity_types, function (EntityTypeInterface $entity_type) {
      return $this->arkContentTypeInfo->isSupportedEntityType($entity_type);
    });
    $options = [];
    foreach ($content_entity_types as $entity_type) {
      $options[$entity_type->id()] = $entity_type->getLabel();
    }

    return $options;
  }

  /**
   * Get all the supported content entity types.
   *
   * @return array
   *   Array of content entity types options, keyed by entity type id.
   */
  private function getContentBundleOptions() {
    // @TODO support more than nodes when we open that up in isSupportedEntityType.
    $options = [];
    $types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    foreach ($types as $type => $info) {
      $options[$type] = $info->label(); // this is more distracting than helpful i think . ' (' . $type . ')';
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\ark\Entity\ArkEnabledContentType $entity */
    $entity = $this->entity;

    $entity->set('id', $entity->getTargetEntityTypeId() . '_' . $entity->getTargetBundle());
    $entity->set('label', 'ARK-enabled ' . $entity->getTargetEntityTypeId() . ' ' . $entity->getTargetBundle());

    $status = $entity->save();

    switch ($status) {
      case SAVED_NEW:
        $this->messenger()->addMessage($this->t('Created the @label configuration.', [
          '@label' => $entity->label(),
        ]));
        break;
    }
    $form_state->setRedirectUrl($entity->toUrl('edit-form'));
    return $status;
  }

}
