<?php

/**
 * @file
 * Contains \Drupal\ark\Form\ArkSettings.
 */

namespace Drupal\ark\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;

class ArkSettings extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ark_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('ark.settings');

    foreach (Element::children($form) as $variable) {
      $config->set($variable, $form_state->getValue($form[$variable]['#parents']));
    }
    $config->save();

    if (method_exists($this, '_submitForm')) {
      $this->_submitForm($form, $form_state);
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['ark.settings'];
  }

  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['ark_naan'] = [
      '#type' => 'textfield',
      '#title' => t('Name Assigning Authority Number'),
      '#description' => t('Before your ARK identifiers can be shared publicly, a NAAN must be provided to you by the <a href="http://www.cdlib.org/">California Digital Library</a>. See <a href="https://wiki.ucop.edu/display/Curation/ARK#ARK-NAAN:theNameAssigningAuthorityNumber">the ARK wiki for more information</a>. Use "pending" here to test your site before you have a number.'),
      '#default_value' =>  \Drupal::config('ark.settings')->get('ark_naan', 'pending'),
      '#size' => 10,
      '#required' => TRUE,
    ];

    $form['ark_name_prefix'] = [
      '#type' => 'textfield',
      '#title' => t('Prefix for ARK names'),
      '#description' => t('Every name assigning authority must have a namespace management strategy. A time-honored technique is to hierarchically partition a namespace into subnamespaces using prefixes that guarantee non-collision of names in different partition. This practice is strongly encouraged for all NAAs, especially when subnamespace management will be delegated to other departments, units, or projects within an organization.'),
      '#default_value' => \Drupal::config('ark.settings')->get('ark_name_prefix', ark_default_prefix()),
      '#size' => 10,
    ];

    $form['ezid_username'] = [
      '#type' => 'textfield',
      '#title' => t('Username for the University of California Digital Library'),
      '#description' => t('Username provided by University of California EZID to use the API system'),
      '#default_value' => \Drupal::config('ark.settings')->get('ezid_username'),
      '#size' => 30,
    ];

    $form['ezid_password'] = [
      '#type' => 'password',
      '#title' => t('Password for the University of California Digital Library'),
      '#description' => t('Password provided by University of California EZID to use the API system'),
      '#default_value' => \Drupal::config('ark.settings')->get('ezid_password'),
      '#size' => 30,
    ];

    $form['remote_server_address'] = [
      '#type' => 'textfield',
      '#title' => t('Remote ARK server address. Normally the server address for the University of California Digital Library'),
      '#description' => t("Server address for the University of California EZID to use the API system, default is 'https://ezid.cdlib.org'."),
      '#default_value' => \Drupal::config('ark.settings')->get('remote_server_address'),
      '#size' => 90,
    ];

    $form['remote_identifier_site_address'] = [
      '#type' => 'textfield',
      '#title' => t('Remote Identifier Site Address. By default "https://n2t.net/"'),
      '#description' => t('Remote Identifier Site Address. The remote Identifier as URL will composed by this server url + the Remote Ark Name value. Ex: https://n2t.net/ark:/99999/fk4pecetest15'),
      '#default_value' => \Drupal::config('ark.settings')->get('remote_identifier_site_address'),
      '#size' => 90,
    ];

    $form['#validate'][] = 'ark_settings_validate';
    $form['#submit'][] = 'pece_ark_settings_submit';

    return parent::buildForm($form, $form_state);
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    if (!ark_naan_format_valid($form_state->getValue(['ark_naan']))) {
      $form_state->setErrorByName('ark_naan', t('NAAN must be a number, or, if you do not yet have one, test the site using "pending".'));
    }
  }

}
?>
